########################################################################
#                                                                      #
#  main.tcl main script for Swumsuit                                   #
#                                             since 2004.12.3          #
#                                             (c) Motomu Nakashima     #
#                                                                      #
########################################################################

# Setup routine. Open windows, packing, etc...
proc MainSetup {} {

    # Project_folder frame
    global Project_folder
    labelframe .project -text $Project_folder -padx 3 -pady 3
    global Create
    button .project.create -text $Create -anchor w \
                                     -command MainCreateFolder
    global Open
    button .project.open   -text $Open -anchor w \
                                     -command MainOpenFolder
    global Save_as
    button .project.save   -text $Save_as -anchor w \
                                     -command MainSaveFolder
    global Info
    button .project.info   -text $Info -anchor w -command ViewComment

    #global Exit EngineExeFile
    #button .project.exit   -text $Exit -anchor w -command exit
    pack \
       .project.create \
       .project.open \
       .project.save \
       .project.info \
       -fill x

    # Open menus
    menu .openm -tearoff false
    global Choose
    .openm add command -label $Choose \
                      -command MainSelectOpenFolder
    .openm add separator
    AddOpenHistory


    # Input frame
    global Input
    labelframe .input -text $Input -padx 3 -pady 3
    global Edit_Body_Geometry
    global Edit_Joint_Motion
    global Load_Body_Geometry
    global Load_Joint_Motion
    #global EditJointMotionExecute
    #global EditBodyGeometryExecute
    global sourcedir
    button .input.geometry  -text $Edit_Body_Geometry  -anchor w \
                       -command EditBodyGeometryExecute
    button .input.motion    -text $Edit_Joint_Motion    -anchor w \
                       -command EditJointMotionExecute
    button .input.lgeometry -text $Load_Body_Geometry  -anchor w \
                            -command LoadBodyGeometry
    button .input.lmotion   -text $Load_Joint_Motion    -anchor w \
                            -command LoadJointMotion
    pack \
       .input.geometry \
       .input.motion \
       .input.lgeometry \
       .input.lmotion \
     -fill x -anchor n

    # Analysis frame
    global Analysis
    global Start_Analysis
    global Edit_Analysis_Settings
    global Load_Analysis_Settings
    labelframe .analysis -text $Analysis -padx 3 -pady 3
    button .analysis.analysis -text $Start_Analysis   -anchor w \
                                  -command StartAnalysis
    button .analysis.edit_setting -text $Edit_Analysis_Settings \
                                           -anchor w \
                                  -command EditAnalysisSettings
    button .analysis.load_setting -text $Load_Analysis_Settings \
                                           -anchor w \
                                  -command LoadAnalysisSettings
    pack \
       .analysis.analysis \
       .analysis.edit_setting \
       .analysis.load_setting \
       -fill x -anchor n

    # Analysis menus
    menu .anam -tearoff false
     global Calculation_settings Initial_condition Output_settings
    .anam add command -label $Calculation_settings \
                      -command EditCalcSettings
    .anam add command -label $Initial_condition \
                      -command EditInitialCondition
    .anam add command -label $Output_settings \
                      -command EditOutputSettings
    global alloreach
    set alloreach all

    # Output frame
    global Output
    global Animation
    global Graph
    labelframe .output -text " $Output " -padx 3 -pady 3
    button .output.animation -text $Animation      -anchor w \
                             -command AnimateStart
    button .output.graph -text $Graph -anchor w \
  -command { DeleteExeFile ; tk_popup .gm [winfo pointerx .] [expr [winfo pointery .]+5]}
    pack \
       .output.animation \
       .output.graph \
       -fill x -anchor n

    # Graph menus
    menu .gm -tearoff false
    global Quantity_Per_Cycle Rigid_Body_Motion Force_Moment_Power \
           Joint_Torque
    .gm add cascade -label $Quantity_Per_Cycle  -menu .gm.q
    .gm add cascade -label $Rigid_Body_Motion   -menu .gm.r
    .gm add cascade -label $Force_Moment_Power -menu .gm.f
    .gm add cascade -label $Joint_Torque        -menu .gm.j

    global Swimming_Speed Stroke_Length Direction Power_Efficiency \
           Input_Power_Per_Cycle Output_Power_Per_Cycle
    menu .gm.q -tearoff false
    .gm.q add command -label $Swimming_Speed \
                      -command "DrawGraph swimming_speed.dat"
    .gm.q add command -label $Stroke_Length \
                      -command "DrawGraph stroke_length.dat"
    .gm.q add command -label $Direction \
                      -command "DrawGraph direction.dat"
    .gm.q add command -label "$Power_Efficiency (1)" \
                      -command "DrawGraph power_efficiency1.dat"
    .gm.q add command -label "$Power_Efficiency (2)" \
                      -command "DrawGraph power_efficiency2.dat"
    .gm.q add command -label $Input_Power_Per_Cycle \
                      -command "DrawGraph input_power.dat"
    .gm.q add command -label "$Output_Power_Per_Cycle (1)" \
                      -command "DrawGraph output_power1.dat"
    .gm.q add command -label "$Output_Power_Per_Cycle (2)" \
                      -command "DrawGraph output_power2.dat"

    menu .gm.r -tearoff false
    global Position_x Position_y Position_z x_y_Locus
    global Velocity_x Velocity_y Velocity_z
    global Angular_Velocity_Axis1 Angular_Velocity_Axis2 \
           Angular_Velocity_Axis3
    global Acceleration_x Acceleration_y Acceleration_z
    global Angular_Acceleration_Axis1 Angular_Acceleration_Axis2 \
           Angular_Acceleration_Axis3
    .gm.r add command -label $Position_x \
                      -command "DrawGraph position_x.dat"
    .gm.r add command -label $Position_y \
                      -command "DrawGraph position_y.dat"
    .gm.r add command -label $Position_z \
                      -command "DrawGraph position_z.dat"
    .gm.r add command -label $x_y_Locus \
                      -command "DrawGraph x-y_locus.dat"
    .gm.r add command -label $Velocity_x \
                      -command "DrawGraph velocity_x.dat"
    .gm.r add command -label $Velocity_y \
                      -command "DrawGraph velocity_y.dat"
    .gm.r add command -label $Velocity_z \
                      -command "DrawGraph velocity_z.dat"
    .gm.r add command -label $Angular_Velocity_Axis1 \
                      -command "DrawGraph angular_velocity_1.dat"
    .gm.r add command -label $Angular_Velocity_Axis2 \
                      -command "DrawGraph angular_velocity_2.dat"
    .gm.r add command -label $Angular_Velocity_Axis3 \
                      -command "DrawGraph angular_velocity_3.dat"
    .gm.r add command -label $Acceleration_x \
                      -command "DrawGraph acceleration_x.dat"
    .gm.r add command -label $Acceleration_y \
                      -command "DrawGraph acceleration_y.dat"
    .gm.r add command -label $Acceleration_z \
                      -command "DrawGraph acceleration_z.dat"
    .gm.r add command -label $Angular_Acceleration_Axis1 \
                      -command "DrawGraph angular_acceleration_1.dat"
    .gm.r add command -label $Angular_Acceleration_Axis2 \
                      -command "DrawGraph angular_acceleration_2.dat"
    .gm.r add command -label $Angular_Acceleration_Axis3 \
                      -command "DrawGraph angular_acceleration_3.dat"

    menu .gm.f -tearoff false
    global Force_x Force_y Force_z \
           Moment_Axis1 Moment_Axis2 Moment_Axis3 \
           Power_x Power_y Power_z
    .gm.f add cascade -label $Force_x      -menu .gm.f.force_x
    .gm.f add cascade -label $Force_y      -menu .gm.f.force_y
    .gm.f add cascade -label $Force_z      -menu .gm.f.force_z
    .gm.f add cascade -label $Moment_Axis1 -menu .gm.f.moment_1
    .gm.f add cascade -label $Moment_Axis2 -menu .gm.f.moment_2
    .gm.f add cascade -label $Moment_Axis3 -menu .gm.f.moment_3
    .gm.f add cascade -label $Power_x      -menu .gm.f.power_x
    .gm.f add cascade -label $Power_y      -menu .gm.f.power_y
    .gm.f add cascade -label $Power_z      -menu .gm.f.power_z

    foreach m {force_x force_y force_z moment_1 moment_2 moment_3 power_x power_y power_z} {
        menu .gm.f.$m -tearoff false
        global Whole_Body Trunk_with_Head Right_Upper_Limb Left_Upper_Limb \
               Right_Lower_Limb Left_Lower_Limb segm_name
        .gm.f.$m add command -label $Whole_Body \
                  -command "DrawGraph ${m}_000whole_body.dat"
        .gm.f.$m add separator
        .gm.f.$m add command -label $Trunk_with_Head \
                  -command "DrawGraph ${m}_001trunk_with_head.dat"
        .gm.f.$m add command -label $Right_Upper_Limb \
                  -command "DrawGraph ${m}_002right_upper_limb.dat"
        .gm.f.$m add command -label $Left_Upper_Limb \
                  -command "DrawGraph ${m}_003left_upper_limb.dat"
        .gm.f.$m add command -label $Right_Lower_Limb \
                  -command "DrawGraph ${m}_004right_lower_limb.dat"
        .gm.f.$m add command -label $Left_Lower_Limb \
                  -command "DrawGraph ${m}_005left_lower_limb.dat"
        .gm.f.$m add separator
        .gm.f.$m add command -label $segm_name(1) \
                  -command "DrawGraph ${m}_01lower_waist.dat"
        .gm.f.$m add command -label $segm_name(2) \
                  -command "DrawGraph ${m}_02upper_waist.dat"
        .gm.f.$m add command -label $segm_name(3) \
                  -command "DrawGraph ${m}_03lower_breast.dat"
        .gm.f.$m add command -label $segm_name(4) \
                  -command "DrawGraph ${m}_04upper_breast.dat"
        .gm.f.$m add command -label $segm_name(5) \
                  -command "DrawGraph ${m}_05shoulder.dat"
        .gm.f.$m add command -label $segm_name(6) \
                  -command "DrawGraph ${m}_06neck.dat"
        .gm.f.$m add command -label $segm_name(7) \
                  -command "DrawGraph ${m}_07head.dat"
        .gm.f.$m add command -label $segm_name(8) \
                  -command "DrawGraph ${m}_08upper_hip.dat"
        .gm.f.$m add command -label $segm_name(9) \
                  -command "DrawGraph ${m}_09lower_hip.dat"
        .gm.f.$m add command -label $segm_name(10) \
                  -command "DrawGraph ${m}_10right_thigh.dat"
        .gm.f.$m add command -label $segm_name(11) \
                  -command "DrawGraph ${m}_11left_thigh.dat"
        .gm.f.$m add command -label $segm_name(12) \
                  -command "DrawGraph ${m}_12right_shank.dat"
        .gm.f.$m add command -label $segm_name(13) \
                  -command "DrawGraph ${m}_13left_shank.dat"
        .gm.f.$m add command -label $segm_name(14) \
                  -command "DrawGraph ${m}_14right_foot.dat"
        .gm.f.$m add command -label $segm_name(15) \
                  -command "DrawGraph ${m}_15left_foot.dat"
        .gm.f.$m add command -label $segm_name(16) \
                  -command "DrawGraph ${m}_16right_upper_arm.dat"
        .gm.f.$m add command -label $segm_name(17) \
                  -command "DrawGraph ${m}_17left_upper_arm.dat"
        .gm.f.$m add command -label $segm_name(18) \
                  -command "DrawGraph ${m}_18right_forearm.dat"
        .gm.f.$m add command -label $segm_name(19) \
                  -command "DrawGraph ${m}_19left_forearm.dat"
        .gm.f.$m add command -label $segm_name(20) \
                  -command "DrawGraph ${m}_20right_hand.dat"
        .gm.f.$m add command -label $segm_name(21) \
                  -command "DrawGraph ${m}_21left_hand.dat"
    }

    menu .gm.j -tearoff false
    .gm.j add cascade -label x  -menu .gm.j.x
    .gm.j add cascade -label y  -menu .gm.j.y
    .gm.j add cascade -label z  -menu .gm.j.z
    .gm.j add separator
    .gm.j add cascade -label xb -menu .gm.j.xb
    .gm.j add cascade -label yb -menu .gm.j.yb
    .gm.j add cascade -label zb -menu .gm.j.zb

    foreach m {x y z xb yb zb} {
        menu .gm.j.$m -tearoff false
        #for {set i 2} {$i<=21} {incr i} {
        #    .gm.j.$m add command -label $segm_name($i)
        #}
        .gm.j.$m add command -label $segm_name(1) \
            -command "DrawGraph joint_torque_${m}_01lower_waist.dat"
        .gm.j.$m add command -label $segm_name(2) \
            -command "DrawGraph joint_torque_${m}_02upper_waist.dat"
        .gm.j.$m add command -label $segm_name(3) \
            -command "DrawGraph joint_torque_${m}_03lower_breast.dat"
        .gm.j.$m add command -label $segm_name(4) \
            -command "DrawGraph joint_torque_${m}_04upper_breast.dat"
        .gm.j.$m add command -label $segm_name(5) \
            -command "DrawGraph joint_torque_${m}_05shoulder.dat"
        .gm.j.$m add command -label $segm_name(6) \
            -command "DrawGraph joint_torque_${m}_06neck.dat"
        .gm.j.$m add command -label $segm_name(7) \
            -command "DrawGraph joint_torque_${m}_07head.dat"
        .gm.j.$m add command -label $segm_name(8) \
            -command "DrawGraph joint_torque_${m}_08upper_hip.dat"
        .gm.j.$m add command -label $segm_name(9) \
            -command "DrawGraph joint_torque_${m}_09lower_hip.dat"
        .gm.j.$m add command -label $segm_name(10) \
            -command "DrawGraph joint_torque_${m}_10right_thigh.dat"
        .gm.j.$m add command -label $segm_name(11) \
            -command "DrawGraph joint_torque_${m}_11left_thigh.dat"
        .gm.j.$m add command -label $segm_name(12) \
            -command "DrawGraph joint_torque_${m}_12right_shank.dat"
        .gm.j.$m add command -label $segm_name(13) \
            -command "DrawGraph joint_torque_${m}_13left_shank.dat"
        .gm.j.$m add command -label $segm_name(14) \
            -command "DrawGraph joint_torque_${m}_14right_foot.dat"
        .gm.j.$m add command -label $segm_name(15) \
            -command "DrawGraph joint_torque_${m}_15left_foot.dat"
        .gm.j.$m add command -label $segm_name(16) \
            -command "DrawGraph joint_torque_${m}_16right_upper_arm.dat"
        .gm.j.$m add command -label $segm_name(17) \
            -command "DrawGraph joint_torque_${m}_17left_upper_arm.dat"
        .gm.j.$m add command -label $segm_name(18) \
            -command "DrawGraph joint_torque_${m}_18right_forearm.dat"
        .gm.j.$m add command -label $segm_name(19) \
            -command "DrawGraph joint_torque_${m}_19left_forearm.dat"
        .gm.j.$m add command -label $segm_name(20) \
            -command "DrawGraph joint_torque_${m}_20right_hand.dat"
        .gm.j.$m add command -label $segm_name(21) \
            -command "DrawGraph joint_torque_${m}_21left_hand.dat"
    }

    # Help frame
    global Help Preferences Manual About Exit
    labelframe .help -text $Help -padx 3 -pady 3
    button .help.option -text $Preferences -anchor w \
                              -command InvokePreferences
    button .help.manual -text $Manual -anchor w -command ManualMessage
    button .help.about  -text $About  -anchor w -command InvokeAbout
    button .help.exit   -text $Exit -anchor w -command exit
    pack \
        .help.option \
        .help.manual \
        .help.about \
        .help.exit \
       -fill x -anchor n

    # Pack all frames
    pack \
        .project \
	.input \
	.analysis \
	.output \
	.help \
        -side left -padx 2 -pady 3 -anchor n

    ChangeIcon .
}


# Create New Project Folder
proc MainCreateFolder {} {
    global dname
    set d [tk_chooseDirectory -initialdir $dname]
    if { [string length $d]!=0 } {
        # mkdir
        file mkdir $d
        # change directory
        set dname $d
        cd $dname
        wm title . "Swumsuit: [pwd]"

        # Add dname to menu and history file
        .openm insert 2 command -label $dname \
                   -command "MainOpenFolderExecute $dname"
        global env Swumsuit_history
        set history ""
        if {[file exists $env(HOME)/$Swumsuit_history]==1} {
            set f [open $env(HOME)/$Swumsuit_history r]
            set history [read $f]
            close $f
        }
        set f [open $env(HOME)/$Swumsuit_history w]
        puts $f $dname
        puts $f $history
        close $f
        # read once again only for 10 lines
        set f [open $env(HOME)/$Swumsuit_history r]
        set history ""
        set line [gets $f]
        set history $history\n$line
        set line2 [gets $f]
        if {$line==$line2} {
            .openm delete 2
        } else {
            set history $history\n$line2
        }
        for {set i 3} {$i<=10} {incr i} {
            set line [gets $f]
            set history $history\n$line
        }
        close $f
        set history [string trim $history]
        # write once again
        set f [open $env(HOME)/$Swumsuit_history w]
        puts $f $history
        close $f
    }
}


# Add Open History
proc AddOpenHistory {} {
    global env Swumsuit_history dname
    if {[file exists $env(HOME)/$Swumsuit_history]==1} {
        set f [open $env(HOME)/$Swumsuit_history r]
        while {[gets $f line]!="-1"} {
            .openm add command -label $line \
              -command "MainOpenFolderExecute $line"
        }
        close $f
    }
}


# Main Open Folder
proc MainOpenFolder {} {
    tk_popup .openm [expr [winfo pointerx .]-5] [expr [winfo pointery .]-5]
}


# Select Project Folder to be Opened
proc MainSelectOpenFolder {} {
    global dname
    set d [tk_chooseDirectory -initialdir $dname]
    MainOpenFolderExecute $d
}

# Open Project Folder
proc MainOpenFolderExecute { d } {
    global dname

    if { [string length $d]!=0 } {
        set dname $d
        cd $dname

        # Add dname to menu and history file
        .openm insert 2 command -label $dname \
                   -command "MainOpenFolderExecute $dname"
        global env Swumsuit_history
        set history ""
        if {[file exists $env(HOME)/$Swumsuit_history]==1} {
            set f [open $env(HOME)/$Swumsuit_history r]
            set history [read $f]
            close $f
        }
        set f [open $env(HOME)/$Swumsuit_history w]
        puts $f $dname
        puts $f $history
        close $f
        # read once again only for 10 lines
        set f [open $env(HOME)/$Swumsuit_history r]
        set history ""
        set line [gets $f]
        set history $history\n$line
        set line2 [gets $f]
        if {$line==$line2} {
            .openm delete 2
        } else {
            set history $history\n$line2
        }
        for {set i 3} {$i<=10} {incr i} {
            set line [gets $f]
            set history $history\n$line
        }
        close $f
        set history [string trim $history]
        # write once again
        set f [open $env(HOME)/$Swumsuit_history w]
        puts $f $history
        close $f

        # Check if necessary *.dat files exist
        # and warn when no file
        if { [file exists body_geometry.dat]==0 } {
            global No_body_geometry_file
            tk_messageBox -type ok -message $No_body_geometry_file \
                          -icon warning
        }
        if { [file exists joint_motion.dat]==0 } {
            global No_joint_motion_file
            tk_messageBox -type ok -message $No_joint_motion_file \
                          -icon warning
        }
        if { [file exists analysis_settings.dat]==0 } {
            global No_analysis_settings_file
            tk_messageBox -type ok -message $No_analysis_settings_file \
                          -icon warning
        }
        wm title . "Swumsuit: [pwd]"

        # open dialog
        global OK Project_folder Was_opened
        tk_messageBox -type ok -icon info \
                -message "$Project_folder [pwd] $Was_opened"

        # delete exe file (if exists)
        DeleteExeFile
        #global EngineExeFile
        #catch { file delete -force $EngineExeFile }
    }
}


# Save Project Folder
proc MainSaveFolder {} {
    global dname

    # get parent directory
    #set e [string last / $dname]
    #if {$e!="-1"} {
    #    set dname2 [string range $dname 0 [expr $e-1]]
    #} else {
    #    set dname2 $dname
    #}

    set d [tk_chooseDirectory -initialdir $dname]
    if { [string length $d]!=0 } {
    if { $d!=$dname } {
        # mkdir folder (if does not exist)
        if { [file exists $d]==0 } {
            file mkdir $d
        }
	# ask if copy comment file
	if {[file exists comment.dat]==1} {
	    global Copy_comment_file
	    set q [tk_messageBox -type yesno -icon question \
	              -message $Copy_comment_file ]
            if {$q=="yes"} {
                file copy -force $dname/comment.dat $d/comment.dat
	    }
	}
        # copy dat files (if exists)
        if { [file exists body_geometry.dat]==1 } {
            file copy -force $dname/body_geometry.dat \
                               $d/body_geometry.dat
        }
        if { [file exists joint_motion.dat]==1 } {
            file copy -force $dname/joint_motion.dat \
                               $d/joint_motion.dat
        }
        if { [file exists analysis_settings.dat]==1 } {
            file copy -force $dname/analysis_settings.dat \
                               $d/analysis_settings.dat
        }
        if { [file exists comment.dat]==1 } {
            file copy -force $dname/comment.dat \
                               $d/comment.dat
        }
        # change directory
        set dname $d
        cd $dname
        wm title . "Swumsuit: [pwd]"

        # Add dname to menu and history file
        .openm insert 2 command -label $dname \
                   -command "MainOpenFolderExecute $dname"
        global env Swumsuit_history
        set history ""
        if {[file exists $env(HOME)/$Swumsuit_history]==1} {
            set f [open $env(HOME)/$Swumsuit_history r]
            set history [read $f]
            close $f
        }
        set f [open $env(HOME)/$Swumsuit_history w]
        puts $f $dname
        puts $f $history
        close $f
        # read once again only for 10 lines
        set f [open $env(HOME)/$Swumsuit_history r]
        set history ""
        set line [gets $f]
        set history $history\n$line
        set line2 [gets $f]
        if {$line==$line2} {
            .openm delete 2
        } else {
            set history $history\n$line2
        }
        for {set i 3} {$i<=10} {incr i} {
            set line [gets $f]
            set history $history\n$line
        }
        close $f
        set history [string trim $history]
        # write once again
        set f [open $env(HOME)/$Swumsuit_history w]
        puts $f $history
        close $f
    }
    }
}


# Load Body Geometry
proc LoadBodyGeometry {} {
    global dname
    set f [tk_getOpenFile -initialdir $dname \
             -filetypes { {"Swumsuit data file" {.dat}} \
                          {"All files" *} } ]
    if { [string length $f]!=0 } {
        file copy -force $f $dname/body_geometry.dat
    }
}


# Load Joint Motion
proc LoadJointMotion {} {
    global dname
    set f [tk_getOpenFile -initialdir $dname \
             -filetypes { {"Swumsuit data file" {.dat}} \
                          {"All files" *} } ]
    if { [string length $f]!=0 } {
        file copy -force $f $dname/joint_motion.dat
    }
}


# Load Analysis Settings
proc LoadAnalysisSettings {} {
    global dname
    set f [tk_getOpenFile -initialdir $dname \
             -filetypes { {"Swumsuit data file" {.dat}} \
                          {"All files" *} } ]
    if { [string length $f]!=0 } {
        file copy -force $f $dname/analysis_settings.dat
    }
}


# Start analysis
proc StartAnalysis {} {
    global Start_analysis_message
    set a [tk_messageBox -type okcancel -default ok -icon question \
            -message $Start_analysis_message]
    if {$a=="ok"} {

        if { [file exists body_geometry.dat]==0 } {
            global No_body_geometry_file
            tk_messageBox -type ok -message $No_body_geometry_file \
                          -icon error
            return
        }
        if { [file exists joint_motion.dat]==0 } {
            global No_joint_motion_file
            tk_messageBox -type ok -message $No_joint_motion_file \
                          -icon error
            return
        }
        if { [file exists analysis_settings.dat]==0 } {
            global No_analysis_settings_file
            tk_messageBox -type ok -message $No_analysis_settings_file \
                          -icon error
            return
        }
 
        # if fluid force coefficients are in old format,
        # correct analysis_setteings.dat
        CorrectAnalysisSettings
        CorrectAnalysisSettings2

        # load analysis settings
        LoadAutoAnalysisSettings
        global vgxsolv vgysolv vgzsolv ome1solv ome2solv ome3solv

        if {$vgxsolv=="0"} {
            if { [file exists input_vgx.dat]==0 } {
                global No_input_vgx_file
                tk_messageBox -type ok -message $No_input_vgx_file \
                              -icon error
                return
            }
        }

        if {$vgysolv=="0"} {
            if { [file exists input_vgy.dat]==0 } {
                global No_input_vgy_file
                tk_messageBox -type ok -message $No_input_vgy_file \
                              -icon error
                return
            }
        }

        if {$vgzsolv=="0"} {
            if { [file exists input_vgz.dat]==0 } {
                global No_input_vgz_file
                tk_messageBox -type ok -message $No_input_vgz_file \
                              -icon error
                return
            }
        }

        if {$ome1solv=="0"} {
            if { [file exists input_ome1.dat]==0 } {
                global No_input_ome1_file
                tk_messageBox -type ok -message $No_input_ome1_file \
                              -icon error
                return
            }
        }

        if {$ome2solv=="0"} {
            if { [file exists input_ome2.dat]==0 } {
                global No_input_ome2_file
                tk_messageBox -type ok -message $No_input_ome2_file \
                              -icon error
                return
            }
        }

        if {$ome3solv=="0"} {
            if { [file exists input_ome3.dat]==0 } {
                global No_input_ome3_file
                tk_messageBox -type ok -message $No_input_ome3_file \
                              -icon error
                return
            }
        }

        if {[file exists Output_data]==1} {
            set ls [glob -nocomplain Output_data/*] 
            if {$ls!=""} {
                eval file delete -force $ls
            }
        } else {
            file mkdir Output_data
        }

        global sourcedir EngineExeFile EngineExeCommand
        file copy -force $sourcedir/Engine/$EngineExeFile [pwd]
        eval $EngineExeCommand
    }
}


# Correct analysis_settings.dat if fluid force coefficients are
# in old format
proc CorrectAnalysisSettings {} {
    set f [open analysis_settings.dat r+]
    set chars [read $f]

    # if new format, do nothing and return
    if {[string first cn_list $chars]!="-1"} {
        return 
    } 

    # if old format, correct data file

    # first, get each value
    set s [string first "cn=" $chars]
    set cn \
       [string trim [lindex [string range $chars [expr $s+3] end] 0]]
 
    set s [string first "ct=" $chars]
    set ct \
       [string trim [lindex [string range $chars [expr $s+3] end] 0]]
 
    set s [string first "ca=" $chars]
    set ca \
       [string trim [lindex [string range $chars [expr $s+3] end] 0]]
 
    # append list lines
    fconfigure $f -translation lf
    puts $f "cn_list="
    for {set i 1} {$i<=21} {incr i} {
    puts $f " $cn"
    }
    puts $f ""

    puts $f "ct_list="
    for {set i 1} {$i<=21} {incr i} {
    puts $f " $ct"
    }
    puts $f ""

    puts $f "ca_list="
    for {set i 1} {$i<=21} {incr i} {
    puts $f " $ca"
    }
    puts $f ""

    close $f
}


# Correct analysis_settings.dat if motsolv are not found
proc CorrectAnalysisSettings2 {} {
    set f [open analysis_settings.dat r+]
    set chars [read $f]

    # if new format, do nothing and return
    if {[string first motsolv $chars]!="-1"} {
        return 
    } 

    # if old format, correct data file

    # append lines
    fconfigure $f -translation lf
    global comsolv vgxsolv vgysolv vgzsolv ome1solv ome2solv ome3solv
    set comsolv 1
    set vgxsolv 1
    set vgysolv 1
    set vgzsolv 1
    set ome1solv 1
    set ome2solv 1
    set ome3solv 1
    puts $f "motsolv="
    puts $f " $comsolv"
    puts $f " $vgxsolv"
    puts $f " $vgysolv"
    puts $f " $vgzsolv"
    puts $f " $ome1solv"
    puts $f " $ome2solv"
    puts $f " $ome3solv\n"
    close $f
}


# Set calculation engine execute
proc SetEngineExe {} {
    global OS Engine_OS sourcedir
    global EngineExeFile EngineExeCommand

    if {$Engine_OS=="Linux"} {
        set EngineExeFile SWUM_ENGINE_LINUX
    } else {
        set EngineExeFile SWUM_ENGINE_WINDOWS.EXE
    }

    if {$OS=="Linux"} {
        set EngineExeCommand "exec xterm -title \"Swumsuit: Analysis\" -e $EngineExeFile &"
    } else {
        set EngineExeCommand "exec cmd.exe /c $EngineExeFile &"
    }
}


# Delete exe file (if exists)
proc DeleteExeFile {} {
    global sourcedir
    set pwd [pwd]
    if {"$pwd"=="$sourcedir/Engine"} { return }
    catch { eval file delete -force [glob SWUM_ENGINE* ] }
}

# Manual message
proc ManualMessage {} {
    global Manual_message
    tk_messageBox -type ok -icon info -message $Manual_message
}


# Invoke "about"
proc InvokeAbout {} {
    global swum_version
    tk_messageBox -type ok -icon info -message "Swumsuit - SWimming hUman Model with Synthetic User Interface Tools\n\nVersion $swum_version"
}


# Joint motion editor
proc EditJointMotionExecute { } {
    global dname sourcedir WISHRUN Locale
    cd $sourcedir
    exec $WISHRUN $sourcedir/edit_joint_motion.tcl $sourcedir $dname $Locale &
    cd $dname
}


# Body geometry editor
proc EditBodyGeometryExecute { } {
    global dname sourcedir WISHRUN Locale
    cd $sourcedir
    exec $WISHRUN $sourcedir/edit_body_geometry.tcl $sourcedir $dname $Locale &
    cd $dname
}


# Animate motion
proc AnimateStart { } {
    DeleteExeFile
    global dname sourcedir WISHRUN Locale
    cd $sourcedir
    exec $WISHRUN $sourcedir/animate_motion.tcl $sourcedir $dname $Locale &
    cd $dname
}



# Change icon
proc ChangeIcon { w } {
    global Platform sourcedir
    if {$Platform=="windows"} {
        update
        wm iconbitmap $w $sourcedir/icon_small.ico
    }
}


# View comment
proc ViewComment { } {
    global dname
    if {[winfo exists .vc ]==0} {
        toplevel .vc
	wm title .vc "Swumsuit: Comment"

        global Project_folder
        labelframe .vc.e -text $Project_folder -padx 3 -pady 3
        #label .vc.e.l -text "${Project_folder}: " 
        entry .vc.e.e -width 80 -state readonly -textvariable dname
        pack  .vc.e.e -fill x

        global Comment
        labelframe .vc.c -text $Comment -padx 3 -pady 3
        text .vc.c.c -width 80 -height 24
        pack .vc.c.c

        frame .vc.p
        global Save Cancel
        button .vc.p.save   -text $Save   -command SaveComment
        button .vc.p.cancel -text $Cancel -command "wm withdraw .vc"
        pack   .vc.p.save .vc.p.cancel -side left -expand 1

        pack .vc.e .vc.c -side top -pady 5
        pack .vc.p -side top -pady 10 -fill x

        ChangeIcon .vc

    } else {
        wm deiconify .vc
    }

    # Load comment
    LoadComment
}


# Load comment
proc LoadComment { } {
    global Locale
    if {[file exists comment.dat]==1} {
        .vc.c.c delete 1.0 end 
        set f [open comment.dat r]
        if {$Locale=="ja_jp"} {
            fconfigure $f -translation crlf -encoding shiftjis
        } else {
            fconfigure $f -translation crlf
        }
        while {[gets $f line]!="-1"} {
            .vc.c.c insert end $line\n
        }
        close $f
        .vc.c.c delete end-1char end
    }
}


# Save comment
proc SaveComment { } {
    global Locale
    set f [open comment.dat w]
    if {$Locale=="ja_jp"} {
        fconfigure $f -translation crlf -encoding shiftjis
    } else {
        fconfigure $f -translation crlf
    }
    puts $f [string trimright [.vc.c.c get 1.0 end]]
    close $f
    wm withdraw .vc
}


# Invoke preferences
proc InvokePreferences { } {
    if {[winfo exists .ip]==0} {
        # open window
        toplevel .ip
        wm title .ip "Swumsuit: Preferences"

        # message
        global You_have_to_restart
        label .ip.m -text $You_have_to_restart

        # Engine_OS
        global Analysis_Engine_OS Auto_Detect
        global Engine_OS_tmp Engine_OS_Detect_tmp
        frame .ip.e -bd 2 -relief groove
        label .ip.e.t -text $Analysis_Engine_OS -anchor w
        checkbutton .ip.e.c -text $Auto_Detect \
                          -variable Engine_OS_Detect_tmp
        bind .ip.e.c <ButtonPress-1> EngineOSDetectToggle
        radiobutton .ip.e.w -text Windows \
                          -variable Engine_OS_tmp -value "Windows NT"
        radiobutton .ip.e.l -text Linux \
                          -variable Engine_OS_tmp -value "Linux"
        pack .ip.e.t -side left
        pack .ip.e.l .ip.e.w .ip.e.c -side right

        # panel
        frame .ip.p
        global OK Cancel
        button .ip.p.ok     -text $OK \
             -command "SavePreferences ; wm withdraw .ip"
        button .ip.p.cancel -text $Cancel \
             -command "wm withdraw .ip"
        pack   .ip.p.ok     \
               .ip.p.cancel \
                 -side left -padx 5

        pack .ip.e -side top -anchor w -fill x
        pack .ip.m -side top -pady 5
        pack .ip.p -side top -pady 5

        ChangeIcon .ip

    } else {
        wm deiconify .ip
    }

    # load preferences
    LoadPreferences
}


# Load preferences
proc LoadPreferences { } {
    global Engine_OS Engine_OS_tmp
    global Engine_OS_Detect Engine_OS_Detect_tmp
    set Engine_OS_tmp $Engine_OS
    set Engine_OS_Detect_tmp $Engine_OS_Detect
    if {$Engine_OS_Detect==1} {
        .ip.e.w configure -state disabled
        .ip.e.l configure -state disabled
    } else {
        .ip.e.w configure -state normal
        .ip.e.l configure -state normal
    }
}


# Save preferences
proc SavePreferences { } {
    global Engine_OS Engine_OS_tmp
    global Engine_OS_Detect Engine_OS_Detect_tmp

    # substitute temporary values
    set Engine_OS $Engine_OS_tmp
    set Engine_OS_Detect $Engine_OS_Detect_tmp
    
    # save to file
    global env Swumsuit_prefs
    set f [open $env(HOME)/$Swumsuit_prefs w]
    puts $f "# Swumsuit preferences \n#\n# This file is created automatically. do not edit!\n"
    if {$Engine_OS_Detect==0} {
        puts $f "set Engine_OS_Detect \"$Engine_OS_Detect\""
        puts $f "set Engine_OS \"$Engine_OS\""
    }
    close $f
}


proc EngineOSDetectToggle { } {
    global Engine_OS_Detect_tmp
    if {$Engine_OS_Detect_tmp==1} {
        # no auto detect
        .ip.e.w configure -state normal
        .ip.e.l configure -state normal
    } else {
        # auto detect
        global Engine_OS_tmp tcl_platform
        set Engine_OS_tmp $tcl_platform(os)
        .ip.e.w configure -state disabled
        .ip.e.l configure -state disabled
    }
}


#########################################################
# Start script
#########################################################

# Set source directory
set sourcedir [file dirname [info script]]
cd $sourcedir
set sourcedir [pwd]

# Set locale
set Locale [lindex $argv 0]

# Window title bar
wm title . "Swumsuit: [pwd]"

# Global initializing
set swum_version "1.4.0"
set dname [pwd]

# Set environment
source set_environment.tcl
# Load message catalog according to locale
source load_catalog.tcl
# Set default font
source set_default_font.tcl

# Select wish
if {$tcl_platform(os)=="Linux"} {
    #set WISHRUN "$sourcedir/wish_linux"
    set WISHRUN "wish"
} else {
    set WISHRUN "$sourcedir/wish_windows.exe"
}


# Load joint motion editor
#set EditJointMotionExecute {cd $sourcedir ; exec $sourcedir/$WISHRUN $sourcedir/edit_joint_motion.tcl $sourcedir $pwd $Locale &}

# Load body geometry editor
#set EditBodyGeometryExecute {cd $sourcedir ; exec $sourcedir/$WISHRUN $sourcedir/edit_body_geometry.tcl $sourcedir $pwd $Locale &}

# Load analysis setting editor
source edit_analysis_settings.tcl

# Load animate motion window
#source animate_motion.tcl

# Load draw graph 
source draw_graph.tcl

# Set calculation engine execute file
SetEngineExe


# Start setup
MainSetup

# Load local script
catch { source ../local.tcl }
